/*
   This program reproduces Figures 3-1a and 3-1b from page 76
   of the book "OpenGL Distilled", by Paul Martz, Addison-Wesley, 2006.

   Notice that the caption to Figure 3-1 is confusing. The caption implies
   that in figure (a) the cylinder is "tranlated first and then rotated"
   when in fact the cylinder is rotated first and then translated (it is
   the coordinate frame that is translated first and then rotated). The
   confusion comes from how we read the OpenGL code, "from top to bottom" or
   "from bottom to top". When we read the OpenGL code "from bottom up", we are
   looking at how the transformations act on the geometry. When we read the
   OpenGL code "from top down", we are looking at how the transformations act
   on coordinate frames. And which transformation you consider as comming "first"
   depends on which direction you are reading the code.

   We get the same distinction when we look at the following expression
   (which coincides with Figure 3-1b)
           R(T(v))
   for the rotation of the translation of the vector v. If we read this expression
   from right to left, we say that first we plot v in the current coordinate frame,
   then translate v to the vector T(v), and then we rotate that vector, R(T(v)).
   If we read the expression from left to right, we say that first we rotate the
   current coordinate frame, then we translate that frame into a new coordinate
   frame, and then we plot v in the newest coordinate frame.


   http://www.opengl-doc.com/Addison.Wesley-OpenGL.Distille/0321336798/ch03lev1sec1.html
*/
#include <windows.h>
#include <math.h>
#include "glut.h"

#define PI 3.14159265


/* Prototype for the display callback function */
void display( void );


int main(int argc, char** argv)
{
   /* Standard GLUT initialization */
   glutInit(&argc,argv);
   glutInitDisplayMode (GLUT_SINGLE | GLUT_RGB); /* default, not needed */
   glutInitWindowSize(500,500);    /* 500 x 500 pixel window */
   glutInitWindowPosition(50,50);  /* place window near top left of display */
   glutCreateWindow("OpenGL Distilled");
   glutDisplayFunc(display);       /* display callback invoked when window opened */

   /* Initialize OpenGL state */
   /* attributes */
   glShadeModel(GL_FLAT);          /* default is GL_SMOOTH */
   glFrontFace(GL_CCW);
   glPolygonMode(GL_FRONT, GL_LINE);
   glPolygonMode(GL_BACK,  GL_LINE);
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */

   /* Enter event loop */
   glutMainLoop();

   return 0;  /* we never get here */
}//main()


/* This function sends one instance of the model down the graphics pipeline. */
void model( void )
{
   int i;
   float angle = 2.0*PI/8.0;

   glBegin(GL_TRIANGLE_FAN);
     glVertex3f(0.0, 0.0, -0.5);
     for (i = 0; i <= 8; i++)
     {
        glVertex3f(0.5*cos(i*angle), 0.5*sin(i*angle), -0.5);
     }
   glEnd();

   glBegin(GL_TRIANGLE_FAN);
     glVertex3f(0.0, 0.0, 0.5);
     for (i = 0; i <= 8; i++)
     {
        glVertex3f(0.5*cos(i*angle), 0.5*sin(i*angle), 0.5);
     }
   glEnd();

   glBegin(GL_LINES);
     for (i = 0; i < 8; i++)
     {
        glVertex3f(0.5*cos(i*angle), 0.5*sin(i*angle), -0.5);
        glVertex3f(0.5*cos(i*angle), 0.5*sin(i*angle),  0.5);
     }
   glEnd();

}//model()


void frame(float gray)
{
   glPushAttrib(GL_ALL_ATTRIB_BITS);
   glColor3f(gray, gray, gray);  // Draw in gray.
   glBegin(GL_LINES);            // Draw a set of axes.
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(1.2, 0.0, 0.0);
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0, 1.2, 0.0);
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0, 0.0, 1.2);
   glEnd();
   glPopAttrib();
}//frame()


/* The display() callback function */
void display( void )
{
   int i;

   glClear(GL_COLOR_BUFFER_BIT);

   // set up viewing
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   glFrustum(-3.0, 3.0, -3.0, 3.0, 2.0, 8.0);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   gluLookAt(1.0, 2.0, 3.5, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);


   // Draw a small coordinate grid.
   glColor3f(0.7, 0.7, 0.7);  // Draw in light gray.
   glBegin(GL_LINES);         // Draw a set of grid lines
     for (i = -2; i <= 2; i++)
     {
        glVertex3f(-2.0, 0.0,    i);
        glVertex3f( 2.0, 0.0,    i);
        glVertex3f(   i, 0.0, -2.0);
        glVertex3f(   i, 0.0,  2.0);
     }
   glEnd();


   frame(0.0);                   // Show the original coordinate frame.

   glColor3f(0.7, 0.7, 1.0);     // Draw in light blue.
   model();                      // Draw the basic (untransformed) model.
   glColor3f(0.0, 0.0, 1.0);     // Draw in blue.

   glPushMatrix();
     glTranslatef(2.0f, 0.0f, 0.0f);       // This is the code for Figure 3-1a.
     glRotatef(-120.0f, 0.0f, 1.0f, 0.0f);
     frame(0.0);                           // Show the transformed coordinate frame.
     model();
   glPopMatrix();

   glPushMatrix();
     glRotatef(-120.0f, 0.0f, 1.0f, 0.0f); // This is the code for Figure 3-1b.
     glTranslatef(2.0f, 0.0f, 0.0f);
     frame(0.0);                           // Show the transformed coordinate frame.
     model();
   glPopMatrix();

   glFlush();

}//display()
